/* $Id$

This file is part of libmspcore
Copyright © 2006 Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#ifndef MSP_CORE_MUTEX_H_
#define MSP_CORE_MUTEX_H_

#include "refptr.h"

namespace Msp {

/**
A class for controlling mutually exclusive access to a resource.  Only one
thread can hold a lock on the mutex at a time.
*/
class Mutex
{
	friend class Semaphore;

private:
	struct Private;

	Private *priv;

public:
	Mutex();
	~Mutex();

	/** Locks the mutex.  If the mutex is already locked, waits until it becomes
	available. */
	void lock();

	/** Attempts to lock the mutex.  Returns true if the lock was acquired,
	otherwise returns false. */
	bool trylock();

	/** Unlocks the mutex. */
	void unlock();
};

/**
Locks the mutex for te lifetime of the object.
*/
class MutexLock
{
private:
	Mutex &mutex;

public:
	MutexLock(Mutex &m, bool l = true): mutex(m) { if(l) mutex.lock(); }
	~MutexLock() { mutex.unlock(); }

	void lock() { mutex.lock(); }
private:
	MutexLock(const MutexLock &);
	MutexLock &operator=(const MutexLock &);
};

/**
Protects a pointer with a mutex.  As long as the MutexPtr (or a copy of it)
exists, the mutex will stay locked.
*/
template<typename T>
class MutexPtr
{
public:
	MutexPtr(T *d, Mutex &m): mutex(new MutexLock(m)), data(d) { }

	T &operator*() const { return *data; }
	T *operator->() const { return data; }
	void clear() { mutex=0; data = 0; }
private:
	RefPtr<MutexLock> mutex;
	T *data;
};

/*template<typename T>
class MutexPtr: public RefCount
{
public:
	MutexPtr(T *d, Mutex &m): mutex(m), data(d) { mutex.lock(); }
	MutexPtr(const MutexPtr<T> &p): RefCount(p), mutex(p.mutex), data(p.data) { }
	T &operator*() const { return *data; }
	T *operator->() const { return data; }
	void clear() { decref(); data = 0; }
	~MutexPtr() { decref(); }
protected:
	Mutex &mutex;
	T     *data;

	bool decref()
	{
		if(!RefCount::decref())
		{
			mutex.unlock();
			return false;
		}
		return true;
	}
};*/

}

#endif
